/*******************************************************************************
 * Copyright (c) 2005 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Bjorn Freeman-Benson - initial implementation
 *     Ward Cunningham - initial implementation
 *******************************************************************************/

package org.eclipse.eclipsemonkey;

import java.io.IOException;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.InvalidRegistryObjectException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.eclipsemonkey.dom.IMonkeyDOMFactory;
import org.eclipse.eclipsemonkey.dom.Utilities;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.RhinoException;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.WrappedException;

public class RunMonkeyScript {
	IWorkbenchWindow window;

	IFile file;

	public RunMonkeyScript(IFile f, IWorkbenchWindow w) {
		this.window = w;
		this.file = f;
	}

	public void run() {
		try {
			Context cx = Context.enter();

			Scriptable sharedScope = null;
			String fileName = this.file.getFullPath().toString();
			Map scriptStore = EclipseMonkeyPlugin.getDefault().getScriptStore();
			StoredScript storedScript = (StoredScript) (scriptStore
					.get(fileName));
			
			if( !storedScript.metadata.ensure_doms_are_loaded(window) )
				return;
			
			String sharedScopeName = storedScript.metadata.getScopeName();
			if (sharedScopeName != null) {
				Map scopeStore = EclipseMonkeyPlugin.getDefault()
						.getScopeStore();
				sharedScope = (Scriptable) scopeStore.get(sharedScopeName);
				if (sharedScope == null) {
					sharedScope = cx.initStandardObjects();
					scopeStore.put(sharedScopeName, sharedScope);
				}
			}

			defineDynamicVariables(file);
			try {

				if (storedScript.compiledScript == null) {
					storedScript.compiledScope = cx.initStandardObjects();
					if (sharedScope != null)
						storedScript.compiledScope.setParentScope(sharedScope);

					defineStandardGlobalVariables(storedScript.compiledScope);
					defineExtensionGlobalVariables(storedScript.compiledScope,
							storedScript.metadata);

					String contents = Utilities.getFileContents(file);
					storedScript.compiledScript = cx.compileString(contents,
							fileName, 1, null);
					storedScript.compiledScript.exec(cx,
							storedScript.compiledScope);
				}

				Object fObj = storedScript.compiledScope.get("main",
						storedScript.compiledScope);
				if (!(fObj instanceof Function)) {
					throw new EvaluatorException(
							"function main() is not defined in ", fileName, 0,
							"", 0);
				} else {
					Object functionArgs[] = {};
					Function f = (Function) fObj;
					f.call(cx, storedScript.compiledScope,
							storedScript.compiledScope, functionArgs);
				}

				// if( sharedScope != null ) {
				// Scriptable a = (Scriptable) sharedScope;
				// Object[] collec = a.getIds();
				//
				// System.out.println("----sharedScope----");
				// for (int i = 0; i < collec.length; i++) {
				// Object object = collec[i];
				// System.out.println(object);
				// }}
				// System.out.println("----scope----");
				// Object[] collec = stored.compiledScope.getIds();
				// for (int i = 0; i < collec.length; i++) {
				// Object object = collec[i];
				// System.out.println(object);
				// }
				// System.out.println("---- ----");

			} finally {
				undefineDynamicVariables(file);
			}
		} catch (WrappedException x) {
			error(x, x.getWrappedException().toString());
		} catch (EvaluatorException x) {
			error(x, x.lineSource() + "\n" + x.details());
		} catch (RhinoException x) {
			error(x, x.details());
		} catch (IOException x) {
			error(x, x.toString());
		} catch (CoreException x) {
			error(x, x.toString());
		} finally {
			Context.exit();
		}

	}

	private void defineDynamicVariables(IFile file) {
		Utilities.state().begin(file);
		Utilities.state().set(Utilities.SCRIPT_NAME,
				file.getFullPath().toPortableString());
	}

	private void undefineDynamicVariables(IFile file) {
		Utilities.state().end(file);
	}

	private void defineStandardGlobalVariables(Scriptable scope) {
		Object wrappedWindow = Context.javaToJS(window, scope);
		ScriptableObject.putProperty(scope, "window", wrappedWindow);

		// Object wrappedWorkspace = Context.javaToJS(ResourcesPlugin
		// .getWorkspace(), scope);
		// ScriptableObject.putProperty(scope, "workspace", wrappedWorkspace);
	}

	private void defineExtensionGlobalVariables(Scriptable scope,
			ScriptMetadata metadata) {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry
				.getExtensionPoint("org.eclipse.eclipsemonkey.dom");
		if (point != null) {
			IExtension[] extensions = point.getExtensions();
			for (int i = 0; i < extensions.length; i++) {
				IExtension extension = extensions[i];
				IConfigurationElement[] configurations = extension
						.getConfigurationElements();
				for (int j = 0; j < configurations.length; j++) {
					IConfigurationElement element = configurations[j];
					try {
						IExtension declaring = element.getDeclaringExtension();
						String declaring_plugin_id = declaring
								.getDeclaringPluginDescriptor()
								.getUniqueIdentifier();
						if (metadata.containsDOM_by_plugin(declaring_plugin_id)) {
							String variableName = element
									.getAttribute("variableName");
							Object object = element
									.createExecutableExtension("class");
							IMonkeyDOMFactory factory = (IMonkeyDOMFactory) object;
							Object rootObject = factory.getDOMroot();
							Object wrappedRoot = Context.javaToJS(rootObject,
									scope);
							ScriptableObject.putProperty(scope, variableName,
									wrappedRoot);
						}
					} catch (InvalidRegistryObjectException x) {
						// ignore bad extensions
					} catch (CoreException x) {
						// ignore bad extensions
					}
				}
			}
		}
	}

	private void error(RhinoException x, String string) {
		MessageDialog.openError(window.getShell(), x.getClass().getName(),
				string + "\n" + x.sourceName() + " #" + x.lineNumber());
	}

	private void error(Exception x, String string) {
		MessageDialog.openError(window.getShell(), x.getClass().getName(),
				string);
	}

}
