/*******************************************************************************
 * Copyright (c) 2005 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Bjorn Freeman-Benson - initial implementation
 *     Ward Cunningham - initial implementation
 *******************************************************************************/

package org.eclipse.eclipsemonkey;

import java.io.IOException;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.InvalidRegistryObjectException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.eclipsemonkey.dom.IMonkeyDOMFactory;
import org.eclipse.eclipsemonkey.dom.Utilities;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.RhinoException;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.WrappedException;

public class RunMonkeyScript {
	IWorkbenchWindow window;

	IFile file;

	ClassLoader old_classloader;

	MonkeyClassLoader classloader;

	protected StoredScript storedScript;

	public RunMonkeyScript(IFile file, IWorkbenchWindow window) {
		this.window = window;
		this.file = file;
	}

	public RunMonkeyScript(IFile file) {
		this.window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		this.file = file;
	}

	public Object run(String entryName, Object[] functionArgs)
			throws RunMonkeyException {
		Object result = null;
		try {
			Context cx = Context.enter();

			Scriptable sharedScope = null;
			String fileName = this.file.getFullPath().toString();
			Map scriptStore = EclipseMonkeyPlugin.getDefault().getScriptStore();
			storedScript = (StoredScript) (scriptStore.get(fileName));

			if (!storedScript.metadata.ensure_doms_are_loaded(window))
				return null;

			String sharedScopeName = storedScript.metadata.getScopeName();
			if (sharedScopeName != null) {
				Map scopeStore = EclipseMonkeyPlugin.getDefault()
						.getScopeStore();
				sharedScope = (Scriptable) scopeStore.get(sharedScopeName);
				if (sharedScope == null) {
					sharedScope = cx.initStandardObjects();
					scopeStore.put(sharedScopeName, sharedScope);
				}
			}

			defineDynamicVariables(file);
			defineClassLoader();
			try {
				boolean needs_compiling = storedScript.compiledScope == null;
				if (needs_compiling) {
					storedScript.compiledScope = cx.initStandardObjects();
					if (sharedScope != null)
						storedScript.compiledScope.setParentScope(sharedScope);
				}
				defineStandardGlobalVariables(storedScript.compiledScope);
				defineExtensionGlobalVariables(storedScript.compiledScope,
						storedScript.metadata);

				if (needs_compiling) {
					String contents = Utilities.getFileContents(file);
					storedScript.compiledScript = cx.compileString(contents,
							fileName, 1, null);
					storedScript.compiledScript.exec(cx,
							storedScript.compiledScope);
					EclipseMonkeyPlugin.getDefault().notifyScriptsChanged();
				}

				Object fObj = storedScript.compiledScope.get(entryName,
						storedScript.compiledScope);
				if (!(fObj instanceof Function)) {
					throw new EvaluatorException("function " + entryName
							+ "() is not defined in ", fileName, 0, "", 0);
				} else {
					Function f = (Function) fObj;
					result = f.call(cx, storedScript.compiledScope,
							storedScript.compiledScope, functionArgs);
				}

			} finally {
				undefineClassLoader();
				undefineDynamicVariables(file);
			}
		} catch (WrappedException x) {
			error(x, x.getWrappedException().toString());
		} catch (EvaluatorException x) {
			error(x, x.lineSource() + "\n" + x.details());
		} catch (RhinoException x) {
			error(x, x.details());
		} catch (IOException x) {
			error(x, this.file.getFullPath().toString(), x.toString());
		} catch (CoreException x) {
			error(x, this.file.getFullPath().toString(), x.toString());
		} finally {
			Context.exit();
		}
		return result;

	}

	private void defineDynamicVariables(IFile file) {
		Utilities.state().begin(file);
		Utilities.state().set(Utilities.SCRIPT_NAME,
				file.getFullPath().toPortableString());
	}

	private void undefineDynamicVariables(IFile file) {
		Utilities.state().end(file);
	}

	private void defineStandardGlobalVariables(Scriptable scope) {
		Object wrappedWindow = Context.javaToJS(window, scope);
		ScriptableObject.putProperty(scope, "window", wrappedWindow);

		// Object wrappedWorkspace = Context.javaToJS(ResourcesPlugin
		// .getWorkspace(), scope);
		// ScriptableObject.putProperty(scope, "workspace", wrappedWorkspace);
	}

	private void defineExtensionGlobalVariables(Scriptable scope,
			ScriptMetadata metadata) {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry
				.getExtensionPoint("org.eclipse.eclipsemonkey.dom");
		if (point != null) {
			IExtension[] extensions = point.getExtensions();
			for (int i = 0; i < extensions.length; i++) {
				IExtension extension = extensions[i];
				IConfigurationElement[] configurations = extension
						.getConfigurationElements();
				for (int j = 0; j < configurations.length; j++) {
					IConfigurationElement element = configurations[j];
					try {
						IExtension declaring = element.getDeclaringExtension();
						String declaring_plugin_id = declaring
								.getDeclaringPluginDescriptor()
								.getUniqueIdentifier();
						if (metadata.containsDOM_by_plugin(declaring_plugin_id)) {
							String variableName = element
									.getAttribute("variableName");
							Object object = element
									.createExecutableExtension("class");
							IMonkeyDOMFactory factory = (IMonkeyDOMFactory) object;
							Object rootObject = factory.getDOMroot();
							ClassLoader rootLoader = rootObject.getClass()
									.getClassLoader();
							classloader.add(rootLoader);
							Object wrappedRoot = Context.javaToJS(rootObject,
									scope);
							ScriptableObject.putProperty(scope, variableName,
									wrappedRoot);
						}
					} catch (InvalidRegistryObjectException x) {
						// ignore bad extensions
					} catch (CoreException x) {
						// ignore bad extensions
					}
				}
			}
		}
	}

	private void error(RhinoException x, String string)
			throws RunMonkeyException {
		throw new RunMonkeyException(x.getClass().getName(), x.sourceName(),
				new Integer(x.lineNumber()), string);
	}

	private void error(Exception x, String fileName, String string)
			throws RunMonkeyException {
		throw new RunMonkeyException(x.getClass().getName(), fileName, null,
				string);
	}

	private void defineClassLoader() {
		classloader = new MonkeyClassLoader();
		old_classloader = Thread.currentThread().getContextClassLoader();
		classloader.add(old_classloader);
		Thread.currentThread().setContextClassLoader(classloader);
	}

	private void undefineClassLoader() {
		Thread.currentThread().setContextClassLoader(old_classloader);
	}
}
