/*******************************************************************************
 * Copyright (c) 2005 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Bjorn Freeman-Benson - initial implementation
 *     Ward Cunningham - initial implementation
 *******************************************************************************/

package org.eclipse.eclipsemonkey;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.Workbench;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.update.search.UpdateSearchRequest;
import org.eclipse.update.search.UpdateSearchScope;
import org.eclipse.update.ui.UpdateJob;
import org.eclipse.update.ui.UpdateManagerUI;
import org.osgi.framework.Bundle;

public class ScriptMetadata {

	private IFile file;

	private String menuName;

	private String scopeName;

	private List doms = new ArrayList();

	public void setMenuName(String string) {
		this.menuName = string;
	}

	public void setFile(IFile file) {
		this.file = file;
	}

	public IFile getFile() {
		return file;
	}

	public String getMenuName() {
		return menuName;
	}

	public String getScopeName() {
		return scopeName;
	}

	public void setScopeName(String s) {
		scopeName = s;
	}

	public List getDOMs() {
		return doms;
	}

	public String getReasonableFilename() {
		if (file != null)
			return file.getName();
		if (menuName != null && !menuName.equals("")) {
			String result = menuName;
			result = result.replaceAll(" ", "_");
			Pattern illegalChars = Pattern.compile("[^\\p{Alnum}_-]");
			Matcher match = illegalChars.matcher(result);
			result = match.replaceAll("");
			if (!result.equals(""))
				return result + ".em";
		}
		return "script.em";
	}

	public boolean containsDOM_by_plugin(String plugin_id) {
		for (Iterator iter = doms.iterator(); iter.hasNext();) {
			DOMDescriptor element = (DOMDescriptor) iter.next();
			if (element.plugin_name.equals(plugin_id))
				return true;
		}
		return false;
	}

	public boolean ensure_doms_are_loaded(IWorkbenchWindow window) {
		String missing_plugin_names = "";
		URLtoPluginMap missing_urls = new URLtoPluginMap();
		for (Iterator iter = doms.iterator(); iter.hasNext();) {
			DOMDescriptor element = (DOMDescriptor) iter.next();
			Bundle b = Platform.getBundle(element.plugin_name);
			if (b == null) {
				missing_plugin_names += "     " + element.plugin_name + "\n";
				missing_urls.add(element);
			} else if (b.getState() == Bundle.UNINSTALLED) {
				missing_plugin_names += "     " + element.plugin_name + "\n";
			}
		}
		missing_plugin_names = missing_plugin_names.substring(0,
				missing_plugin_names.length() - 1);
		if (missing_plugin_names.length() > 0) {
			String choice = notifyMissingDOMs(missing_plugin_names);
			if (choice.startsWith("Install")) {
				launchUpdateInstaller(missing_urls);
			}
			if (choice.startsWith("Edit")) {
				openEditor();
			}
			return false;
		}
		return true;
	}

	class URLtoPluginMap {
		Map map = new HashMap();

		Iterator iterator() {
			return map.keySet().iterator();
		}

		String getPluginNames(String url) {
			Set ids = (Set) map.get(url);
			String idstr = "";
			for (Iterator iterator = ids.iterator(); iterator.hasNext();) {
				String id = (String) iterator.next();
				idstr += id + ", ";
			}
			idstr = idstr.substring(0, idstr.length() - 2);
			return idstr;
		}

		void add(DOMDescriptor domdesc) {
			Set ids = (Set) map.get(domdesc.url);
			if (ids == null)
				ids = new HashSet();
			ids.add(domdesc.plugin_name);
			map.put(domdesc.url, ids);
		}
	}

	private void openEditor() {
		try {
			IWorkbenchPage page = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage();
			IEditorDescriptor desc = PlatformUI.getWorkbench()
					.getEditorRegistry().getDefaultEditor(file.getName());
			if (desc == null) {
				desc = PlatformUI.getWorkbench().getEditorRegistry()
						.getDefaultEditor("foo.txt");
			}
			page.openEditor(new FileEditorInput(file), desc.getId());
		} catch (PartInitException x) {
			MessageDialog.openError(null, "Eclipse::PartInitException",
					"Unable to open editor on " + file.getName() + " due to "
							+ x.toString());
		}
	}

	private void launchUpdateInstaller(URLtoPluginMap missing_urls) {
		UpdateSearchScope scope = new UpdateSearchScope();
		String[] skips = {};
		for (Iterator iter = missing_urls.iterator(); iter.hasNext();) {
			String url = (String) iter.next();
			try {
				String idstr = missing_urls.getPluginNames(url);
				String plural2 = idstr.indexOf(",") >= 0 ? "s" : "";
				scope.addSearchSite("Site providing DOM" + plural2 + " ("
						+ idstr + ")", new URL(url), skips);
			} catch (MalformedURLException x) {
				// ignore
			}
		}
		UpdateSearchRequest request = new UpdateSearchRequest(
				UpdateSearchRequest.createDefaultSiteSearchCategory(), scope);
		UpdateJob job = new UpdateJob("Install Eclipse Monkey DOMs", request);
		Shell shell = Workbench.getInstance().getWorkbenchWindows()[0]
				.getShell();
		UpdateManagerUI.openInstaller(shell, job);
	}

	private String notifyMissingDOMs(String missing_plugin_names) {
		String plural = (missing_plugin_names.indexOf("\n") >= 0 ? "s" : "");
		String[] choices = new String[] { "Cancel Script",
				"Install Plug-in" + plural, "Edit Script" };
		MessageDialog dialog = new MessageDialog(null, "Missing DOM" + plural,
				null, "The script "
						+ this.file.getName()
						+ " requires "
						+ (missing_plugin_names.indexOf("\n") >= 0 ? "these"
								: "this") + " missing DOM plug-in" + plural
						+ ":\n" + missing_plugin_names, MessageDialog.WARNING,
				choices, 1);
		int result = dialog.open();
		String choice = choices[result - 1];
		return choice;
	}

	public static ScriptMetadata getScriptMetadata(String contents) {
		ScriptMetadata metadata = new ScriptMetadata();
		Pattern p = Pattern.compile("^\\s*\\/\\*.*?\\*\\/", Pattern.DOTALL);
		Matcher m = p.matcher(contents);
		if (m.find()) {
			String comment = m.group();
			p = Pattern.compile("Menu:\\s*((\\p{Graph}| )+)", Pattern.DOTALL);
			m = p.matcher(comment);
			if (m.find()) {
				metadata.setMenuName(m.group(1));
			}
			p = Pattern.compile("Scope:\\s*((\\p{Graph}| )+)", Pattern.DOTALL);
			m = p.matcher(comment);
			if (m.find()) {
				metadata.setScopeName(m.group(1));
			}
			p = Pattern.compile("DOM:\\s*(\\p{Graph}+)\\/((\\p{Alnum}|\\.)+)",
					Pattern.DOTALL);
			m = p.matcher(comment);
			while (m.find()) {
				metadata.getDOMs().add(
						new DOMDescriptor(m.group(1), m.group(2)));
			}
		} else {
			// no meta-data comment - do nothing
		}
		return metadata;
	}

}

class DOMDescriptor {
	public String url;

	public String plugin_name;

	DOMDescriptor(String u, String n) {
		url = u;
		plugin_name = n;
	}
}
