/*******************************************************************************
 * Copyright (c) 2006 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Bjorn Freeman-Benson - initial implementation
 *     Ward Cunningham - initial implementation
 *******************************************************************************/

package org.eclipse.eclipsemonkey;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import org.eclipse.core.resources.IFile;
import org.mozilla.javascript.NativeJavaObject;

public class Subscription {

	private final String addMethodName;

	private Object listenerProxy;

	private Method removeMethod;

	private Object source;

	private String sourceString;

	public String trouble = null;

	public Subscription(String _sourceString, String addMethodName) {
		this.sourceString = _sourceString;
		this.addMethodName = addMethodName;
	}

	public void subscribe(IFile script) {
		try {
			RunMonkeyScript runner = new RunMonkeyScript(script);
			Object obj = runner.run(this.sourceString, new Object[0]);
			if (obj instanceof NativeJavaObject) {
				source = ((NativeJavaObject) obj).unwrap();
			} else {
				source = obj;
			}
			subscribe(script, source, addMethodName);
		} catch (Throwable e) {
			trouble(e);
		}
	}

	protected void trouble(Throwable e) {
		if (e instanceof RunMonkeyException) {
			RunMonkeyException x = (RunMonkeyException) e;
			trouble = x.errorMessage + x.optionalLineNumber();
		} else {
			trouble = e.getMessage();
		}
		EclipseMonkeyPlugin.getDefault().notifyScriptsChanged();
	}

	public void unsubscribe() {
		if (trouble != null || removeMethod == null)
			return;
		try {
			removeMethod.invoke(source, new Object[] { listenerProxy });
			removeMethod = null;
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}

	private void subscribe(IFile script, Object foo, String methodName)
			throws InstantiationException, IllegalAccessException,
			InvocationTargetException, NoSuchMethodException {
		InvocationHandler listener = new ScriptSubscriptionListener(script,
				this);
		Method addMethod = findAddMethod(foo, methodName);

		Class listenerType = addMethod.getParameterTypes()[0];
		listenerProxy = Proxy.newProxyInstance(listenerType.getClassLoader(),
				new Class[] { listenerType }, listener);
		addMethod.invoke(foo, new Object[] { listenerProxy });
	}

	private Method findAddMethod(Object source, String methodName)
			throws NoSuchMethodException {
		Method methods[] = source.getClass().getMethods();
		for (int i = 0; i < methods.length; i++) {
			Method m = methods[i];
			if (!(m.getName().equals(methodName)))
				continue;
			if (m.getParameterTypes().length != 1)
				continue;
			if (findRemoveMethod(source, methodName, m.getParameterTypes()) == null)
				continue;

			return m;
		}
		throw new NoSuchMethodException("can't find add method");
	}

	private Method findRemoveMethod(Object source, String methodName,
			Class[] parameterTypes) {
		String removeMethodName = "remove" + methodName.substring(3);
		try {
			removeMethod = source.getClass().getMethod(removeMethodName,
					parameterTypes);
			return removeMethod;
		} catch (Exception e) {
			return null;
		}
	}

	class ScriptSubscriptionListener implements InvocationHandler {
		IFile script;
		Subscription subscription;

		public ScriptSubscriptionListener(IFile script,
				Subscription subscription) {
			this.script = script;
			this.subscription = subscription;
		}

		public Object invoke(Object proxy, Method method, Object[] args)
				throws Throwable {
			/*
			 * use RunMonkeyScript instead of MenuRunMonkeyScript because we do
			 * NOT want the ctrl-shift-M "re-run the last script" accelerator
			 * bound to this script.
			 */
			try {
				RunMonkeyScript runner = new RunMonkeyScript(script);
				return runner.run(method.getName(), args);
			} catch (RunMonkeyException x) {
				subscription.unsubscribe();
				subscription.trouble(x);
				return null;
			}
		}

	}

	public String toString() {
		return (trouble == null ? "" : trouble + ": ") + this.sourceString
				+ "()." + this.addMethodName
				+ (removeMethod == null ? " (not listening)" : "");
	}
}