/**
 * Copyright (c) 2005-2006 Aptana, Inc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html. If redistributing this code,
 * this entire header must remain intact.
 */
package org.eclipse.eclipsemonkey.lang.javascript.doms.global;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.lang.reflect.Method;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.Platform;
import org.eclipse.eclipsemonkey.lang.javascript.JavaScriptClassLoader;
import org.eclipse.eclipsemonkey.lang.javascript.JavaScriptGlobal;
import org.eclipse.eclipsemonkey.lang.javascript.JavaScriptThread;
import org.eclipse.eclipsemonkey.utils.StringUtils;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.FunctionObject;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.osgi.framework.Bundle;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

public class Global extends ScriptableObject
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -6433207559862280811L;
	
	private HashMap _runningSetTimeouts = new HashMap();
	private int _setTimeoutIndex;
	
	public Global(Scriptable scope)
	{
		this.setParentScope(scope);
		
		// make list of functions
		String[] names = {
			"alert", //$NON-NLS-1$
			"clearTimeout", //$NON-NLS-1$
			"confirm", //$NON-NLS-1$
			"execute", //$NON-NLS-1$
			"getProperty", //$NON-NLS-1$
			"loadBundle",
			"parseXML", //$NON-NLS-1$
			"prompt", //$NON-NLS-1$
			"runOnUIThread", //$NON-NLS-1$
			"setTimeout", //$NON-NLS-1$
		};
		
		if(scope instanceof JavaScriptGlobal)
		{
			defineFunctionProperties(names, Global.class, READONLY | PERMANENT, (ScriptableObject) scope);
		}
	}

	public void defineFunctionProperties(String[] names, Class clazz, int attributes, ScriptableObject target)
	{
		for (int i=0; i < names.length; i++) {
			String name = names[i];
			
	        Method m[] = clazz.getDeclaredMethods();
			
	        for (int j = 0; j < m.length; j++) {
				Method mm = m[j];
				
				if(mm.getName().equals(name))
				{
					FunctionObject f = new FunctionObject(name, mm, this);
					target.defineProperty(name, f, attributes);
				}
			}
		}
	}
	/*
	 * Methods
	 */

	/**
	 * Execute a command-line in the system shell
	 * 
	 * @param cx
	 * @param thisObj
	 * @param args
	 *            The string command to execute
	 * @param funObj
	 * @return Returns an object array where the first object is the return code, the second object is the text from
	 *         stdout, and the third object is the text from stderr
	 */
	public static Scriptable execute(Context cx, Scriptable thisObj, Object[] args, Function funObj)
	{
		Scriptable result = null;

		if (args.length > 0)
		{
			String command = args[0].toString();
			String input = StringUtils.EMPTY;

			if (args.length > 1)
			{
				input = args[1].toString();
			}

			Process p = null;
			int retCode = 0;
			String stdout = StringUtils.EMPTY;
			String stderr = StringUtils.EMPTY;

			try
			{
				p = Runtime.getRuntime().exec(command);
				p.getOutputStream().write(input.getBytes());
				p.getOutputStream().flush();
				p.getOutputStream().close();
				retCode = p.waitFor();
				// retCode = p.exitValue();
				stdout = getText(p.getInputStream());
				stderr = getText(p.getErrorStream());

			}
			catch (IOException e)
			{
				if (p != null)
				{
					retCode = p.exitValue();
				}
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}

			// create Object
			Scriptable scope = ScriptableObject.getTopLevelScope(thisObj);

			result = cx.newObject(scope, "Object"); //$NON-NLS-1$

			// set property values
			result.put("code", result, new Integer(retCode)); //$NON-NLS-1$
			result.put("stdout", result, stdout); //$NON-NLS-1$
			result.put("stderr", result, stderr); //$NON-NLS-1$
		}

		// return result
		return result;
	}

	
	/**
	 * Popup an alert box with the specified message
	 * 
	 * @param message
	 *            The message to display in an alert dialog
	 */
	public void alert(final String message)
	{
		final Display currentDisplay = Display.getCurrent();

		if (currentDisplay != null)
		{
			currentDisplay.syncExec(new Runnable()
			{
				public void run()
				{
					Shell shell = currentDisplay.getActiveShell();

					if (shell != null)
					{
						MessageDialog.openWarning(shell, "Eclipse Monkey Alert", message); //$NON-NLS-1$
					}
				}
			});
		}
	}

	/**
	 * Popup an confirm box with the specified message
	 * 
	 * @param message
	 *            The message to display at the confirmation prompt
	 * @return boolean
	 */
	public boolean confirm(final String message)
	{
		/**
		 * inner class for result
		 */
		class Answer
		{
			public boolean result = false;
		}

		// create instance of inner class
		final Answer a = new Answer();

		// get reply from user
		final Display currentDisplay = Display.getCurrent();

		if (currentDisplay != null)
		{
			currentDisplay.syncExec(new Runnable()
			{
				public void run()
				{
					Shell shell = currentDisplay.getActiveShell();

					if (shell != null)
					{
						a.result = MessageDialog.openConfirm(shell, "Eclipse Monkey Confirm", message); //$NON-NLS-1$
					}
				}
			});
		}

		return a.result;
	}

	/*
	 * Methods
	 */
	
	/**
	 * loadBundle
	 * 
	 * @param cx
	 * @param thisObj
	 * @param args
	 * @param funObj
	 */
	public static void loadBundle(Context cx, Scriptable thisObj, Object[] args, Function funObj)
	{
		if (args.length > 0)
		{
			String bundleName = Context.toString(args[0]);
			Bundle bundle = Platform.getBundle(bundleName);
	
			if (bundle == null)
			{
				throw new RuntimeException("Global_Bundle_Not_Found: "  + bundleName);
			}
			
			ClassLoader c = cx.getApplicationClassLoader();
			
			if(c instanceof JavaScriptClassLoader)
			{
				JavaScriptClassLoader classLoader = (JavaScriptClassLoader) c;
				classLoader.addBundle(bundle);
			}
			else
			{
				throw new RuntimeException("JavaScriptClassLoader not the application Classloader.");
			}
		}
	}

	/**
	 * Popup an prompt box with the specified message
	 * 
	 * @param cx
	 * @param thisObj
	 * @param args
	 *            The message to display in the prompt dialog
	 * @param funObj
	 * @return boolean
	 */
	public static Object prompt(Context cx, Scriptable thisObj, Object[] args, Function funObj)
	{
		final Display currentDisplay = Display.getCurrent();
		String messageArg = "Prompt";
		String defaultValueArg = "";

		if (args.length > 0)
		{
			messageArg = Context.toString(args[0]);
		}
		if (args.length > 1)
		{
			defaultValueArg = Context.toString(args[1]);
		}

		/**
		 * Answer
		 */
		class Answer
		{
			public Object result = "";
		}

		final String message = messageArg;
		final String defaultValue = defaultValueArg;
		final Answer a = new Answer();

		if (currentDisplay != null)
		{
			currentDisplay.syncExec(new Runnable()
			{

				public void run()
				{
					Shell shell = currentDisplay.getActiveShell();

					if (shell != null)
					{
						InputDialog dialog = new InputDialog(null, "Eclipse Monkey Prompt", message, defaultValue, null); //$NON-NLS-1$
						int dialogResult = dialog.open();
						if (dialogResult == Window.OK)
						{
							a.result = dialog.getValue();
						}
						else
						{
							a.result = Context.getUndefinedValue();
						}
					}
				}
			});
		}

		return a.result;
	}

	/**
	 * runOnUIThread
	 * 
	 * @param cx
	 * @param thisObj
	 * @param args
	 * @param funObj
	 */
	public static void runOnUIThread(Context cx, Scriptable thisObj, Object[] args, Function funObj)
	{
		if (args.length > 0 && args[0] instanceof Function)
		{
			Function f = (Function) args[0];

			// get display
			final IWorkbench workbench = PlatformUI.getWorkbench();
			Display display = workbench.getDisplay();

			// execute callback in the correct thread
			display.asyncExec(new JavaScriptThread(f.getParentScope(), f, new Object[0]));
		}
	}

	/**
	 * parseXML
	 * 
	 * @param xml
	 * @return Document
	 */
	public Document parseXML(String xml)
	{
		Document result = null;

		if (xml != null && xml.length() > 0)
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder;
			try
			{
				builder = factory.newDocumentBuilder();
				// parse XML to create our document element
				InputStream in = new ByteArrayInputStream(xml.getBytes());

				result = builder.parse(in);
			}
			catch (ParserConfigurationException e)
			{
				e.printStackTrace();
			}
			catch (SAXException e)
			{
				e.printStackTrace();
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
		}

		return result;
	}

	/**
	 * Get a list of the full paths of all files in the Active Library view
	 * 
	 * @param property
	 *            The system property name to retrieve
	 * @return Returns a string array of full paths
	 */
	public String getProperty(String property)
	{
		String result = null;

		if (property != null && property.length() > 0)
		{
			result = System.getProperty(property, "undefined"); //$NON-NLS-1$
		}

		return result;
	}
	
	/**
	 * setTimeout
	 * 
	 * @param f
	 * @param timeout
	 * @return int
	 */
	public int setTimeout(final Function f, final int timeout)
	{
		final Global self = this;
		final int timeoutIndex = self._setTimeoutIndex++;
	
		Thread thread = new Thread(new Runnable()
		{
			public void run()
			{
				boolean active = true;
	
				try
				{
					// sleep
					Thread.sleep(timeout);
	
					// check if this was canceled while sleeping
					synchronized (self._runningSetTimeouts)
					{
						active = self._runningSetTimeouts.containsKey(new Integer(timeoutIndex));
					}
	
					// call the associated function if this is still active
					if (active)
					{
						// get display
						final IWorkbench workbench = PlatformUI.getWorkbench();
	
						if (workbench == null)
						{
							return;
						}
	
						Display display = workbench.getDisplay();
	
						if (display == null || display.isDisposed())
						{
							return;
						}
	
						// execute callback in the correct thread
						display.asyncExec(new Runnable()
						{
							public void run()
							{
								Scriptable scope = f.getParentScope();
								Context cx = Context.enter();
	
								f.call(cx, scope, scope, new Object[0]);
	
								Context.exit();
	
								self.clearTimeout(timeoutIndex);
							}
						});
					}
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}, "Eclipse Monkey JavaScript setTimeout"); //$NON-NLS-1$
	
		// save reference to this thread to indicate that it should run. Note that clearTimeout will
		// remove this entry
		// and if that is down before the thread wakes, then the action in the thread will not
		// execute
		this._runningSetTimeouts.put(new Integer(timeoutIndex), thread);
	
		thread.setDaemon(true);
	
		// start the thread
		thread.start();
	
		// return the timeout index in case the thread wants to abort this timeout
		return timeoutIndex;
	}

	/**
	 * clearTimeout
	 * 
	 * @param timeoutId
	 */
	public void clearTimeout(int timeoutId)
	{
		synchronized (this._runningSetTimeouts)
		{
			Integer id = new Integer(timeoutId);

			if (this._runningSetTimeouts.containsKey(id))
			{
				this._runningSetTimeouts.remove(id);
			}
		}
	}

	public String getClassName() {
		return "Global";
	}

	/**
	 * Get all text from the given input stream
	 * 
	 * @param stream
	 *            The input stream
	 * @return Returns all text from the input stream
	 * @throws IOException
	 */
	protected static String getText(InputStream stream) throws IOException
	{
		// create output buffer
		StringWriter sw = new StringWriter();
	
		// read contents into a string buffer
		try
		{
			// get buffered reader
			InputStreamReader isr = new InputStreamReader(stream);
			BufferedReader reader = new BufferedReader(isr);
	
			// create temporary buffer
			char[] buf = new char[1024];
	
			// fill buffer
			int numRead = reader.read(buf);
	
			// keep reading until the end of the stream
			while (numRead != -1)
			{
				// output temp buffer to output buffer
				sw.write(buf, 0, numRead);
	
				// fill buffer
				numRead = reader.read(buf);
			}
		}
		finally
		{
			if (stream != null)
			{
				stream.close();
			}
		}
	
		// return string buffer's content
		return sw.toString();
	}

	

}
